import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.List;
import java.util.Vector;

import javax.sound.midi.MidiDevice;


import com.frinika.audio.AudioContext;
import com.frinika.midi.MidiDebugDevice;
import com.frinika.project.ProjectContainer;
import com.frinika.sequencer.gui.mixer.SynthWrapper;
import com.frinika.sequencer.model.Lane;
import com.frinika.sequencer.model.MidiLane;
import com.frinika.synth.SynthRack;
import com.frinika.synth.synths.MySampler;

import junit.framework.TestCase;
/*
 * Created on Jun 13, 2006
 *
 * Copyright (c) 2004-2006 Peter Johan Salomonsen (http://www.petersalomonsen.com)
 * 
 * http://www.frinika.com
 * 
 * This file is part of Frinika.
 * 
 * Frinika is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Frinika is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Frinika; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * Test setting up multiple mididevices mapped to different lanes. This test checks the lane
 * mapping after a reload
 * @author Peter Johan Salomonsen
 */
public class TestGearSavingLoading extends TestCase {
    ProjectContainer proj;

    Vector<SynthRack> synthRacks = new Vector<SynthRack>();;
    Vector<MySampler> samplers = new Vector<MySampler>();
    Vector<MidiDebugDevice> mDevices = new Vector<MidiDebugDevice>();
    
    byte[] savedProject;
    
	protected void setUp() throws Exception {
        super.setUp();
        
        new AudioContext(); // Initialize the Audio system
        proj = new ProjectContainer();
        
        /**
         * The test will check that the ones below appear in the same order
         */
        createDebugMidi();
        createGearAndSampler();
        createDebugMidi();
        createDebugMidi();
        createGearAndSampler();
        createGearAndSampler();
        /**
         * ---------------------------------------------------------------
         */
        
        // Save project in a bytearray
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        
        ObjectOutputStream oos = new ObjectOutputStream(baos);
        oos.writeObject(proj);
    
        savedProject = baos.toByteArray();

		// And reopen project from stored bytearray
		ByteArrayInputStream bais = new ByteArrayInputStream(savedProject);
		ObjectInputStream ois = new ObjectInputStream(bais);
		proj = (ProjectContainer)ois.readObject();

	}

	void createDebugMidi() throws Exception
	{
		MidiDebugDevice dev = new MidiDebugDevice();
		mDevices.add(dev);
		addLaneAndMidiDevice(dev);
	}
	
	void createGearAndSampler() throws Exception
	{
		SynthRack dev = new SynthRack(AudioContext.getDefaultAudioContext().getVoiceServer(), null);
		synthRacks.add(dev);
        samplers.add(new MySampler(dev));
        dev.setSynth(1,samplers.get(samplers.size()-1));
        addLaneAndMidiDevice(dev);
	}
	
	void addLaneAndMidiDevice(MidiDevice dev) throws Exception
	{
        proj.getSequencer().addMidiOutDevice(dev);
        MidiLane lane = proj.createMidiLane();
        lane.setMidiDevice(dev);
        lane.setMidiChannel(1);
	}
	
	/**
	 * Test that lanes are mapped correctly
	 * @throws Exception
	 */
	public void testLaneMapping() throws Exception
	{
		List<Lane> lanes = proj.getLanes().get(0).getChildren();
		assertEquals(SynthWrapper.class,((MidiLane)lanes.get(1)).getMidiDevice().getClass());
		assertEquals(SynthRack.class,((MidiLane)lanes.get(2)).getMidiDevice().getClass());
		assertEquals(SynthWrapper.class,((MidiLane)lanes.get(3)).getMidiDevice().getClass());
		assertEquals(SynthWrapper.class,((MidiLane)lanes.get(4)).getMidiDevice().getClass());
		assertEquals(SynthRack.class,((MidiLane)lanes.get(5)).getMidiDevice().getClass());
		assertEquals(SynthRack.class,((MidiLane)lanes.get(6)).getMidiDevice().getClass());
	}
}
