import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import javax.sound.midi.ShortMessage;
import javax.sound.midi.Track;

import com.frinika.audio.AudioContext;
import com.frinika.project.ProjectContainer;
import com.frinika.sequencer.FrinikaTrackWrapper;
import com.frinika.sequencer.model.MidiLane;
import com.frinika.sequencer.model.MidiPart;
import com.frinika.sequencer.model.NoteEvent;
import com.frinika.synth.SynthRack;
import com.frinika.synth.synths.MySampler;

import junit.framework.TestCase;
/*
 * Created on Apr 23, 2006
 *
 * Copyright (c) 2005 Peter Johan Salomonsen (http://www.petersalomonsen.com)
 * 
 * http://www.frinika.com
 * 
 * This file is part of Frinika.
 * 
 * Frinika is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * Frinika is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Frinika; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * @author Peter Johan Salomonsen
 */
public class TestSaveAndLoadProject extends TestCase {
    MidiPart part;
    ProjectContainer proj;
    MySampler sampler;
    
    protected void setUp() throws Exception {
        super.setUp();
                
        proj = new ProjectContainer();
        
        // Create a lane
        com.frinika.sequencer.model.MidiLane lane = proj.createMidiLane();

        // Create a MidiPart
        part = new MidiPart(lane);
        
        // Add a note
        
        NoteEvent noteEvent = new NoteEvent(part, 0,60, 100, 0, 128);
        part.add(noteEvent);
        
        // Create a softsynth instance
        
        new AudioContext(); // Initialize the Audio system
        
        // Initialize a synth (MySampler)
        SynthRack synthRack = new SynthRack(AudioContext.getDefaultAudioContext().getVoiceServer(), null);
        sampler = new MySampler(synthRack);
        synthRack.setSynth(1,sampler);
        
        // Add a dummy synthrack to confuse
        proj.getSequencer().addMidiOutDevice(new SynthRack(AudioContext.getDefaultAudioContext().getVoiceServer(), null));
        // Now add the real synthrack
        proj.getSequencer().addMidiOutDevice(synthRack);
        
        /**
         * Open a drumkit soundfont
         */
        sampler.getImporter().getSoundFont(new File("soundfonts/Club.SF2"));
        sampler.getImporter().getInstrument(0);
        
        // Assign the synth to first lane
        
        lane.getTrack().setMidiDevice(synthRack);
        lane.getTrack().setMidiChannel(1);
    }
    
    public void testSaveAndLoad()
    {
        
        ByteArrayOutputStream bstm = new ByteArrayOutputStream();
        try {

            checkMidiSequence();
            
            ObjectOutputStream objStm = new ObjectOutputStream(bstm);
            objStm.writeObject(proj);
            
            byte[] saved = bstm.toByteArray();
            
            ByteArrayInputStream inbstm = new ByteArrayInputStream(saved);
            
            ObjectInputStream objInStm = new ObjectInputStream(inbstm);
            
            checkMidiSequence();
            
            // Check that the synthrack is restored correctly
            FrinikaTrackWrapper ftw = ((MidiLane)proj.getProjectLane().getChildren().get(1)).getTrack();
            assertEquals(ftw.getMidiChannel(),1);
            assertEquals(ftw.getMidiDevice() instanceof SynthRack,true);
            assertEquals(((SynthRack)ftw.getMidiDevice()).getSynth(1) instanceof MySampler,true);
        
            assertEquals(((SynthRack)ftw.getMidiDevice()).getSynth(1).getInstrumentName(),sampler.getInstrumentName());
        } catch (Exception e) {
            e.printStackTrace();
            fail(e.getMessage());
        }
    }

    /**
     * Check the MIDI contents
     *
     */
    private void checkMidiSequence() {
        System.out.println("CheckMidiSequence");
        Track track = proj.getSequence().getTracks()[2];
        assertEquals(track.get(3).getTick(),0);
        
        assertEquals(true, track.get(3).getMessage() instanceof ShortMessage);
        ShortMessage msg = (ShortMessage)track.get(3).getMessage();
        assertEquals(msg.getCommand(),ShortMessage.NOTE_ON);
        assertEquals(msg.getData1(),60);
        assertEquals(msg.getData2(),100);
       
        assertEquals(track.get(4).getTick(),128);
        assertEquals(true, track.get(4).getMessage() instanceof ShortMessage);
        msg = (ShortMessage)track.get(4).getMessage();
        assertEquals(msg.getCommand(),ShortMessage.NOTE_ON);
        assertEquals(msg.getData1(),60);
        assertEquals(msg.getData2(),0);
        
        
    }
}

